@@ -1,7 +1,9 @@
+from agentmake import AGENTMAKE_USER_DIR
 AGENTMAKE_CONFIG = {
     "print_on_terminal": False,
     "word_wrap": False,
 }
 OLLAMA_NOT_FOUND = "`Ollama` is not found! BibleMate AI uses `Ollama` to generate embeddings for semantic searches. You may install it from https://ollama.com/ so that you can perform semantic searches of the Bible with BibleMate AI."
 def fix_string(content):
-    return content.replace(" ", " ").replace("‑", "-")
\ No newline at end of file
+    return content.replace(" ", " ").replace("‑", "-")
+BIBLEMATEDATA = os.path.join(AGENTMAKE_USER_DIR, "biblemate", "data")
\ No newline at end of file
