@@ -7,4 +7,5 @@ default_bible="NET"
 max_semantic_matches=15
 max_log_lines=2000
 mcp_port=33333
-disabled_tools=[]
\ No newline at end of file
+embedding_model="paraphrase-multilingual"
+disabled_tools=['uba']
\ No newline at end of file
