@@ -2,10 +2,10 @@ from biblemate.core.systems import *
 from biblemate.ui.prompts import getInput
 from biblemate.ui.info import get_banner
 from biblemate.ui.selection_dialog import TerminalModeDialogs
-from biblemate import config, AGENTMAKE_CONFIG, OLLAMA_NOT_FOUND, fix_string
-from biblemate.core.bible_db import BibleVectorDatabase
+from biblemate import config, AGENTMAKE_CONFIG, OLLAMA_NOT_FOUND, fix_string, BIBLEMATEDATA
+from biblemate.uba.bible import BibleVectorDatabase
 from pathlib import Path
-import asyncio, re, os, subprocess, click, shutil, pprint, argparse
+import asyncio, re, os, subprocess, click, shutil, pprint, argparse, json
 from copy import deepcopy
 from alive_progress import alive_bar
 from fastmcp import Client
@@ -27,7 +27,7 @@ set_log_file_max_lines(log_file, config.max_log_lines)
 
 # bible data
 builtin_bible_data = os.path.join(os.path.dirname(os.path.realpath(__file__)), "data", "bibles")
-user_bible_data = os.path.join(AGENTMAKE_USER_DIR, "biblemate", "data", "bibles")
+user_bible_data = os.path.join(BIBLEMATEDATA, "bibles")
 if not os.path.isdir(user_bible_data):
     Path(user_bible_data).mkdir(parents=True, exist_ok=True)
 user_bible = os.path.join(user_bible_data, f"{config.default_bible}.bible")
@@ -131,7 +131,15 @@ Get a static text-based response directly from a text-based AI model without usi
         }
         prompts_schema[p.name] = schema
     
-    return tools, tools_schema, master_available_tools, available_tools, tool_descriptions, prompts, prompts_schema
+    resources_raw = await client.list_resources()
+    resources = {r.name: r.description for r in resources_raw}
+    resources = dict(sorted(resources.items()))
+
+    templates_raw = await client.list_resource_templates()
+    templates = {r.name: r.description for r in templates_raw}
+    templates = dict(sorted(templates.items()))
+    
+    return tools, tools_schema, master_available_tools, available_tools, tool_descriptions, prompts, prompts_schema, resources, templates
 
 def backup_conversation(console, messages, master_plan):
     """Backs up the current conversation to the user's directory."""
@@ -167,6 +175,7 @@ default_bible="{config.default_bible}"
 max_semantic_matches={config.max_semantic_matches}
 max_log_lines={config.max_log_lines}
 mcp_port={config.mcp_port}
+embedding_model="{config.embedding_model}"
 disabled_tools={pprint.pformat(config.disabled_tools)}"""
     writeTextFile(config_file, configurations)
 
@@ -191,13 +200,16 @@ async def main_async():
     dialogs = TerminalModeDialogs(None)
 
     async with client:
-        tools, tools_schema, master_available_tools, available_tools, tool_descriptions, prompts, prompts_schema = await initialize_app(client)
+        tools, tools_schema, master_available_tools, available_tools, tool_descriptions, prompts, prompts_schema, resources, templates = await initialize_app(client)
         write_user_config() # remove the temporary `config.backend`
         
         available_tools_pattern = "|".join(available_tools)
         prompt_list = [f"/{p}" for p in prompts.keys()]
         prompt_pattern = "|".join(prompt_list)
         prompt_pattern = f"""^({prompt_pattern}) """
+        template_list = [f"//{t}/" for t in templates.keys()]
+        template_pattern = "|".join(template_list)
+        template_pattern = f"""^({template_pattern})"""
 
         user_request = ""
         master_plan = ""
@@ -270,7 +282,7 @@ async def main_async():
                 ".mode": "change AI mode",
                 ".tools": "list available tools",
                 ".plans": "list available plans",
-                #".resources": "list available resources", # TODO explore relevant usage for this project
+                ".resources": "list available resources",
                 ".promptengineer": "toggle auto prompt engineering",
                 ".lite": "toggle lite context",
                 ".steps": "configure the maximum number of steps",
@@ -280,7 +292,7 @@ async def main_async():
                 ".open": "open a file or directory",
                 ".help": "help page",
             }
-            input_suggestions = list(action_list.keys())+[f"@{t} " for t in available_tools]+prompt_list
+            input_suggestions = list(action_list.keys())+["@ ", "@@ "]+[f"@{t} " for t in available_tools]+[f"{p} " for p in prompt_list]+[f"//{r}" for r in resources.keys()]+template_list # "" is for generating ideas
             user_request = await getInput("> ", input_suggestions)
             while not user_request.strip():
                 # Generate ideas for `prompts to try`
@@ -298,6 +310,54 @@ async def main_async():
                 # Get input agin
                 user_request = await getInput("> ", input_suggestions)
 
+            # display resources
+            if user_request.startswith("//") and user_request[2:] in resources:
+                resource = user_request[2:]
+                resource_content = await client.read_resource(f"resource://{resource}")
+                if hasattr(resource_content[0], 'text'):
+                    console.rule()
+                    resource_text = resource_content[0].text
+                    if resource_text.startswith("{"):
+                        resource_dict = json.loads(resource_text)
+                        display_content = "\n".join([f"- `{k}`: {v}" for k, v in resource_dict.items()])
+                    else:
+                        display_content = resource_text
+                    resource_description = resources.get(resource, "")
+                    console.print(Markdown(f"## Resource: `{resource.capitalize()}`\n\n{resource_description}\n\n{display_content}"))
+                    console.rule()
+                continue
+
+            # run templates
+            if re.search(template_pattern, user_request):
+                try:
+                    uri = re.sub("^(.*?)/", r"\1://", user_request[2:])
+                    resource_content = await client.read_resource(uri)
+                    resource_content = resource_content[0].text
+                    while resource_content.startswith("[") and resource_content.endswith("]"):
+                        options = json.loads(resource_content)
+                        select = await dialogs.getMultipleSelection(
+                            options=options,
+                            title="Multiple Matches",
+                            text="Select one of them to continue:"
+                        )
+                        if select:
+                            resource_content = await client.read_resource(re.sub("^(.*?/)[^/]*?$", r"\1", uri)+select)
+                            resource_content = resource_content[0].text
+                        else:
+                            resource_content = "Cancelled by user."
+                    if resource_content:
+                        messages += [
+                            {"role": "user", "content": f"Retrieve resource from:\n\n{uri}"},
+                            {"role": "assistant", "content": resource_content},
+                        ]
+                        console.rule()
+                        console.print(Markdown(resource_content))
+                        console.rule()
+                    continue
+                except Exception as e: # invalid uri
+                    print(f"Error: {e}\n")
+                    continue
+            
             # system command
             if user_request == ".open":
                 user_request = f".open {os.getcwd()}"
@@ -343,15 +403,21 @@ async def main_async():
                     )
                     if enabled_tools is not None:
                         available_tools = enabled_tools
+                        available_tools_pattern = "|".join(available_tools) # reset available tools pattern
                         config.disabled_tools = [i for i in master_available_tools if not i in available_tools]
                         write_user_config()
                     console.rule()
                     tools_descriptions = [f"- `{name}`: {description}" for name, description in tools.items()]
                     console.print(Markdown("## Available Tools\n\n"+"\n".join(tools_descriptions)))
                     console.rule()
+                elif user_request == ".resources":
+                    console.rule()
+                    resources_descriptions = [f"- `//{name}`: {description}" for name, description in resources.items()]
+                    console.print(Markdown("## Available Resources\n\n"+"\n".join(resources_descriptions)))
+                    console.rule()
                 elif user_request == ".plans":
                     console.rule()
-                    prompts_descriptions = [f"- `{name}`: {description}" for name, description in prompts.items()]
+                    prompts_descriptions = [f"- `/{name}`: {description}" for name, description in prompts.items()]
                     console.print(Markdown("## Available Plans\n\n"+"\n".join(prompts_descriptions)))
                     console.rule()
                 elif user_request == ".backend":
@@ -475,7 +541,7 @@ async def main_async():
                 console.print(Markdown(f"# User Request\n\n{user_request}\n\n# Master plan\n\n{master_plan}"))
 
             # Prompt Engineering
-            if not specified_tool == "@@" and config.prompt_engineering:
+            if not specified_tool == "@@" and not specified_tool == "uba" and config.prompt_engineering:
                 async def run_prompt_engineering():
                     nonlocal user_request
                     try:
